/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.3)            

   class      : ScopeModel
   date       : 28.12.97
   description: a model for the selected scope
   modified   : 19.02.98 - start and end date for selection added

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class ScopeModel extends Observable {

  // class variable declarations
  public final static int DAYLY   = 0;
  public final static int WEEKLY  = 1;
  public final static int MONTHLY = 2;

  // instance variable declarations
  private Calendar calendar;
  private Date curDate;
  private int scopeWidth;
  private Date startOfScope;
  private Date endOfScope;
  private String user;
  private CalendarEntry selectedObject;
  private Date selectedStart;
  private Date selectedEnd;
  private boolean dateChangedFlag;
  private boolean scopeChangedFlag;
  private boolean userChangedFlag;
  private boolean selObChangedFlag;
  // constructors
  public ScopeModel() {
    init();
  } // end constructor ScopeModel

  // methods
  private void init() {
    calendar     = CalendarCreator.get();
    curDate      = extractDay(new Date());
    scopeWidth   = DAYLY;
    startOfScope = new Date(curDate.getTime());
    calendar.setTime(startOfScope);
    calendar.add(Calendar.DATE,1);
    endOfScope       = new Date(calendar.getTime().getTime());
    user             = null;
    selectedObject   = null;
    selectedStart    = null;
    selectedEnd      = null;
    dateChangedFlag  = false;
    scopeChangedFlag = false;
    userChangedFlag  = false;
    selObChangedFlag = false;
  } // end init

  public void setScopeWidth(int scopeWidth) {
    dateChangedFlag  = false;
    scopeChangedFlag = false;
    userChangedFlag  = false;
    selObChangedFlag = false;
    if (internalSetScopeWidth(scopeWidth)) {
      scopeChangedFlag = true;
      notify(null);
    } // end if
  } // end setScopeWidth

  public int getScopeWidth() {
    return scopeWidth;
  } // end getScopeWidth

  private boolean internalSetScopeWidth(int scopeWidth) {
    long ts = startOfScope.getTime();
    long te = endOfScope.getTime();
    this.scopeWidth = scopeWidth;
    startOfScope.setTime(curDate.getTime());
    calendar.setTime(startOfScope);
    switch (this.scopeWidth) {
      case MONTHLY:
        calendar.add(Calendar.DATE,-calendar.get(Calendar.DAY_OF_MONTH)+1);
        startOfScope.setTime(calendar.getTime().getTime());
        calendar.add(Calendar.MONTH,1);
        endOfScope.setTime(calendar.getTime().getTime());
      break;
      case WEEKLY:
        int firstDay = calendar.getFirstDayOfWeek();
        while (calendar.get(Calendar.DAY_OF_WEEK) != firstDay)
          calendar.add(Calendar.DATE,-1);
        // end while
        startOfScope.setTime(calendar.getTime().getTime());
        calendar.add(Calendar.DATE,7);
        endOfScope.setTime(calendar.getTime().getTime());
      break;
      default: // DAYLY
        calendar.add(Calendar.DATE,1);
        endOfScope.setTime(calendar.getTime().getTime());
    } // end switch
    return ((ts != startOfScope.getTime()) || (te != endOfScope.getTime()));
  } // end internalSetScopeWidth

  public Date getStartOfScope() {
    return startOfScope;
  } // end getStartOfScope

  public Date getEndOfScope() {
    return endOfScope;
  } // end getEndOfScope

  public void setDate(Date date) {
    setDate(date,null);
  } // end setDate

  public void setDate(Date date, Object arg) {
    dateChangedFlag  = false;
    scopeChangedFlag = false;
    userChangedFlag  = false;
    selObChangedFlag = false;
    long t           = curDate.getTime();
    curDate          = extractDay(date);
    dateChangedFlag  = (t != curDate.getTime());
    if ((dateChangedFlag) &&
        ((curDate.before(startOfScope)) ||
         (curDate.after(endOfScope)) ||
         (curDate.equals(endOfScope)))) {
      scopeChangedFlag = internalSetScopeWidth(scopeWidth);
    } // end if
    if (dateChangedFlag) notify(arg);
  } // end setDate

  public Date getDate() {
    return curDate;
  } // end getDate

  public boolean matchesScope(Date date) {
    return ((!date.before(startOfScope)) &&
            (date.before(endOfScope)));
  } // end matchesScope

  public boolean matchesScope(CalendarEntry entry) {
    return ((entry.getStart().before(endOfScope)) &&
            (entry.getEnd().after(startOfScope)));
  } // end matchesScope

  private Date extractDay(Date date) {
    Date d = new Date((date.getTime()/1000)*1000); // cut of milliseconds

    calendar.setTime(d);
    calendar.add(Calendar.SECOND,-calendar.get(Calendar.SECOND));
    calendar.add(Calendar.MINUTE,-calendar.get(Calendar.MINUTE));
    calendar.add(Calendar.HOUR,-calendar.get(Calendar.HOUR_OF_DAY));
    return calendar.getTime();
  } // end extractDay

  public void setUser(String user) {
    dateChangedFlag  = false;
    scopeChangedFlag = false;
    selObChangedFlag = false;
    userChangedFlag  = (!user.equals(this.user));
    if (userChangedFlag) {
      this.user = user;
      notify(null);
    } // end if
  } // end setUser

  public String getUser() {
    return user;
  } // end getUser

  public void setSelectedObject(Object obj) {
    setSelectedObject(obj,null,null);
  } // end setSelectedObject

  public void setSelectedObject(Object obj, Date start, Date end) {
    dateChangedFlag  = false;
    scopeChangedFlag = false;
    userChangedFlag  = false;
    selObChangedFlag = false;
    CalendarEntry t  = selectedObject;
    if ((obj != null) && (obj instanceof CalendarEntry))
      selectedObject = (CalendarEntry)obj;
    else
      selectedObject = null;
    // end if
    selectedStart = start;
    selectedEnd   = end;
    if ((t == null) && (selectedObject == null)) return;
    if (((t == null) && (selectedObject != null)) ||
        ((t != null) && (selectedObject == null))) {
      selObChangedFlag = true;
      notify(null);
      return;
    } // end if
    if (selObChangedFlag = (!t.equals(selectedObject))) notify(null);
  } // end setSelectedObject

  public CalendarEntry getSelectedObject() {
    return selectedObject;
  } // end getSelectedObject

  public Date getSelectedStart() {
    return selectedStart;
  } // end getSelectedStart

  public Date getSelectedEnd() {
    return selectedEnd;
  } // end getSelectedEnd

  public boolean dateChanged() {
    return dateChangedFlag;
  } // end dateChanged

  public boolean scopeChanged() {
    return scopeChangedFlag;
  } // end scopeChanged

  public boolean userChanged() {
    return userChangedFlag;
  } // end userChanged

  public boolean selObChanged() {
    return selObChangedFlag;
  } // end selObChanged

  private void notify(Object arg) {
    Assert.out.println("ScopeModel: "+
      "date "+(dateChangedFlag?"changed":"n.c.")+
      ", scope "+(scopeChangedFlag?"changed":"n.c.")+
      ", user "+(userChangedFlag?"changed":"n.c.")+
      ", selection "+(selObChangedFlag?"changed":"n.c.")); 
    setChanged();
    notifyObservers(arg);
  } // end notify

} // end ScopeModel

/* ******************************************************************
   end of file
*********************************************************************/
