/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : WebDatabase
   date       : 16.10.97
   description: a class that implements calendar database operations
                over the web
   modified   : 26.02.98 - all methods made synchronized

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.io.*;
import java.net.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

class WebDatabase implements CalendarDatabase {

  // instance variable declarations
  private Socket s;
  private DataInputStream in;
  private DataOutputStream out;
  private Vector outVector;
  private Vector inVector;
  private String errStr;

  // constructors
  public WebDatabase(Socket s) throws IOException {
    this.s    = s;
    in        = new DataInputStream(s.getInputStream());
    out       = new DataOutputStream(s.getOutputStream());
    outVector = new Vector();
    inVector  = new Vector();
    errStr    = "";
  } // end constructor WebDatabase

  // methods
  public synchronized boolean failed() {
  // returns true if the last database operation failed

    return !errStr.equals("");
  } // end failed

  public synchronized String getErrorMsg() {
  // returns a string describing the last error

    if (errStr.equals("")) return null;
      else return errStr;
  } // end getErrorMsg

  public synchronized void login(String user, String passwd) {
    String cypheredPasswd = (new PasswdCypher()).cypher(passwd);

    outVector.removeAllElements();
    outVector.addElement(new RequestHeader(RequestHeader.LOGIN,user,
                                           cypheredPasswd));
    if (sendVector() && receiveVector()) {
      errStr = ((ReplyHeader)inVector.elementAt(0)).getErrorMsg();
    } // end if
  } // end login

  public synchronized void logout() {

    outVector.removeAllElements();
    outVector.addElement(new RequestHeader(RequestHeader.LOGOUT));
    if (sendVector() && receiveVector()) {
      errStr = ((ReplyHeader)inVector.elementAt(0)).getErrorMsg();
    } // end if
  } // end logout

  public synchronized void changePasswd(String newPasswd) {
    String cypheredPasswd = (new PasswdCypher()).cypher(newPasswd);

    outVector.removeAllElements();
    outVector.addElement(new RequestHeader(RequestHeader.CHANGEPASSWD,
                         cypheredPasswd));
    if (sendVector() && receiveVector()) {
      errStr = ((ReplyHeader)inVector.elementAt(0)).getErrorMsg();
    } // end if
  } // end changePasswd

  public synchronized String[] getUsers() {

    outVector.removeAllElements();
    outVector.addElement(new RequestHeader(RequestHeader.GETUSERS));
    if (sendVector() && receiveVector()) {
      errStr = ((ReplyHeader)inVector.elementAt(0)).getErrorMsg();
      return ((ReplyHeader)inVector.elementAt(0)).getUsers();
    }
    else {
      return null;
    } // end if
  } // end getUsers

  public synchronized Enumeration getDates(String user, Date from, Date to,
                              boolean periodical) {
    Vector resultVector;

    outVector.removeAllElements();
    outVector.addElement(new RequestHeader(RequestHeader.GETDATES,user,
                         from,to,periodical));
    if (sendVector() && receiveVector()) {
      errStr = ((ReplyHeader)inVector.elementAt(0)).getErrorMsg();
      resultVector = (Vector)inVector.clone();
      resultVector.removeElementAt(0);
      return resultVector.elements();
    } // end if
    return null;
  } // end getDates

  public synchronized Enumeration getMessages() {
  // reads new dates written from other users (destructive read)
    Vector resultVector;

    outVector.removeAllElements();
    outVector.addElement(new RequestHeader(RequestHeader.GETMESSAGES));
    if (sendVector() && receiveVector()) {
      errStr = ((ReplyHeader)inVector.elementAt(0)).getErrorMsg();
      resultVector = (Vector)inVector.clone();
      resultVector.removeElementAt(0);
      return resultVector.elements();
    } // end if
    return null;
  } // end getMessages

  public synchronized void putDate(CalendarEntry cEntry) {

    outVector.removeAllElements();
    outVector.addElement(new RequestHeader(RequestHeader.PUTDATE));
    cEntry.setLastInSeq(true);
    outVector.addElement(cEntry);
    if (sendVector() && receiveVector()) {
      errStr = ((ReplyHeader)inVector.elementAt(0)).getErrorMsg();
    } // end if
  } // end putDate

  public synchronized void deleteDate(CalendarEntry cEntry, boolean forAll) {

    outVector.removeAllElements();
    outVector.addElement(new RequestHeader(RequestHeader.DELETEDATE,forAll));
    cEntry.setLastInSeq(true);
    outVector.addElement(cEntry);
    if (sendVector() && receiveVector()) {
      errStr = ((ReplyHeader)inVector.elementAt(0)).getErrorMsg();
    } // end if
  } // end deleteDate

  public synchronized void updateDate(CalendarEntry oldEntry,
                         CalendarEntry newEntry, boolean forAll) {

    outVector.removeAllElements();
    outVector.addElement(new RequestHeader(RequestHeader.UPDATEDATE,forAll));
    oldEntry.setLastInSeq(false);
    outVector.addElement(oldEntry);
    newEntry.setLastInSeq(true);
    outVector.addElement(newEntry);
    if (sendVector() && receiveVector()) {
      errStr = ((ReplyHeader)inVector.elementAt(0)).getErrorMsg();
    } // end if
  } // end updateDate

  public synchronized void endSession() {
  // closes all connections, frees all resources

    outVector.removeAllElements();
    outVector.addElement(new RequestHeader(RequestHeader.ENDSESSION));
    if (sendVector() && receiveVector()) {
      errStr = ((ReplyHeader)inVector.elementAt(0)).getErrorMsg();
    } // end if
  } // end endSession

  private boolean sendVector() {
    // writes all elements of the outVector to the out stream and returns
    // true if the operation was successfull
    boolean result = true;
    int n          = outVector.size();
    int i;

    try {
      for (i=0; i<n; i++) {
        ((Streamable)outVector.elementAt(i)).outputTo(out);
      } // end for
    } catch (IOException excep) {
      errStr = "Error during network write operation"+
               "\n"+excep.toString();
      result = false;
    }
    try {
      out.flush();
    } catch (IOException excep) {
      errStr = "Error during network flush operation"+
               "\n"+excep.toString();
      result = false;
    }
    return result;
  } // end sendVector

  private boolean receiveVector() {
    // reads the reply header and attached objects from the in stream and
    // returns true if the operation was successfull
    boolean result  = true;
    boolean isFirst = true;

    inVector.removeAllElements();
    try {
      do {
        if (isFirst) {
          inVector.addElement(new ReplyHeader(in));
          isFirst = false;
        }
        else {
          inVector.addElement(new CalendarEntry(in));
        } // end if
      } while (!((Streamable)inVector.elementAt(inVector.size()-1)).isLastInSeq());
    } catch (IOException excep) {
      errStr = "Error during network read operation"+
               "\n"+excep.toString();
      result = false;
    }
    return result;
  } // end receiveVector

} // end WebDatabase

/* ******************************************************************
   end of file
*********************************************************************/
